@@ -4,13 +4,13 @@ algo <- 2
 
 test_that("OwenQ2 for R=0 equals ptOwen", {
   skip_on_os("mac")
-  owenQ2 <- sapply(1:8, function(nu) OwenQ2(nu=nu, t=2, delta=2, R=0, algo))
+  owenQ2 <- sapply(1:8, function(nu) OwenQ2(nu=nu, t=2, delta=2, R=1e-17, algo))
   owenS <- sapply(1:8, function(nu) ptOwen(q=2, nu=nu, delta=2))
   expect_equal(owenQ2, owenS, tolerance=1e-15)
 })
 
 test_that("OwenQ2 is correctly vectorized", {
-  delta <- c(Inf, 0, -Inf); R <- c(1,2,3)
+  delta <- c(Inf, 0.01, -Inf); R <- c(1,2,3)
   nu <- 5; t <- 2
   expect_identical(OwenQ2(nu, t, delta, R, algo),
                    c(OwenQ2(nu, t, delta[1], R[1], algo),
